local Recount2_Configs = Recount2.Configs

local ViewConfig
local CurrentView

local function SetFontSize(string, size)
	local Font, Height, Flags = string:GetFont()
	string:SetFont(Font, size, Flags)
end


--Tabs
--1. Tags
--2. Filters
--3. Src Filter
--4. Dst Filter
--5. Grouping

local function CreateMainTab()
	local tab = ViewConfig.Tabs:AddTab("Main")
	

	ViewConfig.viewName = Recount2.Widgets:CreateEditBox(tab, 250, 14, '', 'Name')
	ViewConfig.viewName:SetPoint("TOPLEFT", ViewConfig.innerFrame, "TOPLEFT", 60, -20)
	ViewConfig.barValue = Recount2.Widgets:CreateEditBox(tab, 250, 14, '', 'Bar Value')
	ViewConfig.barValue:SetPoint("TOPLEFT", ViewConfig.innerFrame, "TOPLEFT", 60, -35)
	ViewConfig.leftText = Recount2.Widgets:CreateEditBox(tab, 250, 14, '', 'Left Text')
	ViewConfig.leftText:SetPoint("TOPLEFT", ViewConfig.innerFrame, "TOPLEFT", 60, -50)
	ViewConfig.rightText = Recount2.Widgets:CreateEditBox(tab, 250, 14, '', 'Right Text')
	ViewConfig.rightText:SetPoint("TOPLEFT", ViewConfig.innerFrame, "TOPLEFT", 60, -65)

	CreateTagsTab = nil
end

--Tables
local consts = Recount2.Const
local translations = Recount2.TranslationTables

--Filters

local function CreateLabel(parent, text)
	local label = parent:CreateFontString(nil,"OVERLAY","GameFontHighlightSmall")
	label:SetJustifyH("LEFT")
	label:SetText(text)
	label:SetTextColor(1,1,0.2,1)
	Recount2:AddFontString(label)
	SetFontSize(label, 13)
	return label
end

local function CreateFilterTab()
	local tab = ViewConfig.Tabs:AddTab("Filters")

	local label = CreateLabel(tab, "Info Filters")
	label:SetPoint("TOPLEFT", ViewConfig.innerFrame, "TOPLEFT", 0, -20)

	local i = 0
	for key, text in pairs(Recount2.GroupNames.Info) do
		 local checkbox= Recount2.Widgets:CreateCheckbox(tab, 75, 14, text, "View_Info", key)
		 checkbox:SetPoint("TOPLEFT", label, "BOTTOMLEFT", 0, -i*14)
		 i = i + 1
	end

	label = CreateLabel(tab, "Type Filters")
	label:SetPoint("TOPLEFT", ViewConfig.innerFrame, "TOPLEFT", 79, -20)

	i = 0
	for key, text in pairs(Recount2.GroupNames.Type) do
		 local checkbox= Recount2.Widgets:CreateCheckbox(tab, 75, 14, text, "View_Type", key)
		 checkbox:SetPoint("TOPLEFT", label, "BOTTOMLEFT", 0, -i*14)
		 i = i + 1
	end

	label = CreateLabel(tab, "School Filters")
	label:SetPoint("TOPLEFT", ViewConfig.innerFrame, "TOPLEFT", 158, -20)

	i = 0
	for key, text in pairs(Recount2.GroupNames.School) do
		 local checkbox= Recount2.Widgets:CreateCheckbox(tab, 75, 14, text, "View_School", key)
		 checkbox:SetPoint("TOPLEFT", label, "BOTTOMLEFT", 0, -i*14)
		 i = i + 1
	end

	label = CreateLabel(tab, "Power Filters")
	label:SetPoint("TOPLEFT", ViewConfig.innerFrame, "TOPLEFT", 237, -20)

	i = 0
	for key, text in pairs(Recount2.GroupNames.Power) do
		 local checkbox= Recount2.Widgets:CreateCheckbox(tab, 75, 14, text, "View_Power", key)
		 checkbox:SetPoint("TOPLEFT", label, "BOTTOMLEFT", 0, -i*14)
		 i = i + 1
	end
	
	CreateFilterTab = nil
end

local function CreateCheckboxList(tab, which, title, constLookups, translations, xPos, yPos)
	local label = CreateLabel(tab, title)
	label:SetPoint("TOPLEFT", ViewConfig.innerFrame, "TOPLEFT", xPos, -20+yPos)

	for i, text in ipairs(constLookups) do
		 local checkbox= Recount2.Widgets:CreateCheckboxList(tab, 75, 14, translations[text], "View_"..which..title, text)
		 checkbox:SetPoint("TOPLEFT", label, "BOTTOMLEFT", 0, -(i-1)*14)
	end
end

local function SetupCombatantTab(tab, which)
	CreateCheckboxList(tab, which, "Class", consts.ClassNames, translations.ClassNames, 0, 0)
	CreateCheckboxList(tab, which, "Race", consts.Race, translations.Race, 79, 0)
	CreateCheckboxList(tab, which, "Faction", consts.Faction, translations.Faction, 158, 0)
	CreateCheckboxList(tab, which, "Affiliation", consts.Affiliation, translations.Affiliation, 237, 0)
	CreateCheckboxList(tab, which, "Reaction", consts.Reaction, translations.Reaction, 0, -170)
end

local function CreateSourceTab()
	local tab = ViewConfig.Tabs:AddTab("Source")

	SetupCombatantTab(tab, "Src")

	CreateSourceTab = nil
end

local function CreateDestinationTab()
	local tab = ViewConfig.Tabs:AddTab("Target")

	SetupCombatantTab(tab, "Dst")

	SetupCombatantTab = nil
	CreateDestinationTab = nil
end

local function CreateGroupingTab()
	local tab = ViewConfig.Tabs:AddTab("Grouping")

	for k, group in ipairs(consts.Groups) do
		local xOffset = (k-1)*105
		for k2, groupOn in ipairs(group) do
			local radio = Recount2.Widgets:CreateRadioButton(tab, 100, 14, translations.Groups[groupOn], "View_GroupOn", groupOn)
			radio:SetPoint("TOPLEFT", ViewConfig.innerFrame, xOffset, -20-(k2-1)*14)
		end
	end

	CreateGroupingTab = nil
end

local function NotEmpty(t)
	for _, _ in pairs(t) do
		return true
	end
	return false
end

local function SaveView()
	local view = CurrentView

	view.name = ViewConfig.viewName:GetText()
	ViewConfig:SetTitle("Configuring View - "..view.name)
	view.barValue = ViewConfig.barValue:GetText()
	view.leftText = ViewConfig.leftText:GetText()
	view.rightText = ViewConfig.rightText:GetText()

	srcClass = Recount2.Widgets:GetListCheckValue("View_SrcClass")
	srcRace  = Recount2.Widgets:GetListCheckValue("View_SrcRace")
	srcFaction = Recount2.Widgets:GetListCheckValue("View_SrcFaction")
	srcAffiliation = Recount2.Widgets:GetListCheckValue("View_SrcAffiliation")
	srcReaction = Recount2.Widgets:GetListCheckValue("View_SrcReaction")

	if NotEmpty(srcClass) or NotEmpty(srcRace) or NotEmpty(srcFaction) or NotEmpty(srcAffiliation) or NotEmpty(srcReaction) then
		view.srcCombatant = {}
		if NotEmpty(srcClass) then
			view.srcCombatant.class = srcClass
		end
		if NotEmpty(srcRace) then
			view.srcCombatant.race = srcRace
		end
		if NotEmpty(srcFaction) then
			view.srcCombatant.faction = srcFaction
		end
		if NotEmpty(srcAffiliation) then
			view.srcCombatant.affiliation = srcAffiliation
		end
		if NotEmpty(srcReaction) then
			view.srcCombatant.reaction = srcReaction
		end
	else
		view.srcCombatant = nil
	end

	dstClass = Recount2.Widgets:GetListCheckValue("View_DstClass")
	dstRace = Recount2.Widgets:GetListCheckValue("View_DstRace")
	dstFaction =Recount2.Widgets:GetListCheckValue("View_DstFaction")
	dstAffiliation = Recount2.Widgets:GetListCheckValue("View_DstAffiliation")
	dstReaction = Recount2.Widgets:GetListCheckValue("View_DstReaction")

	if NotEmpty(dstClass) or NotEmpty(dstRace) or NotEmpty(dstFaction) or NotEmpty(dstAffiliation) or NotEmpty(dstReaction) then
		view.dstCombatant = {}
		if NotEmpty(dstClass) then
			view.dstCombatant.class = dstClass
		end
		if NotEmpty(dstRace) then
			view.dstCombatant.race = dstRace
		end
		if NotEmpty(dstFaction) then
			view.dstCombatant.faction = dstFaction
		end
		if NotEmpty(dstAffiliation) then
			view.dstCombatant.affiliation = dstAffiliation
		end
		if NotEmpty(dstReaction) then
			view.dstCombatant.reaction = dstReaction
		end
	else
		view.dstCombatant = nil
	end

	view.infoFilter = Recount2.Widgets:GetCompositeCheckValue("View_Info")
	view.infoFilter = view.infoFilter~=0 and view.infoFilter or nil
	view.typeFilter = Recount2.Widgets:GetCompositeCheckValue("View_Type")
	view.typeFilter = view.typeFilter~=0 and view.typeFilter or nil
	view.schoolFilter = Recount2.Widgets:GetCompositeCheckValue("View_School")
	view.schoolFilter = view.schoolFilter~=0 and view.schoolFilter or nil
	view.powerFilter = Recount2.Widgets:GetCompositeCheckValue("View_Power")
	view.powerFilter = view.powerFilter~=0 and view.powerFilter or nil

	view.groupOn = Recount2.Widgets:GetRadioValue("View_GroupOn")

	for _, v in pairs(Recount2.mainListWindows) do
		v:ResetCurFilter()
	end
end

local function LoadView()
	local view = CurrentView
	ViewConfig:SetTitle("Configuring View - "..view.name)

	ViewConfig.viewName:SetText(view.name)
	ViewConfig.barValue:SetText(view.barValue)
	ViewConfig.leftText:SetText(view.leftText)
	ViewConfig.rightText:SetText(view.rightText)

	Recount2.Widgets:SetListCheckValue("View_SrcClass",   view.srcCombatant and view.srcCombatant.class or {})
	Recount2.Widgets:SetListCheckValue("View_SrcRace",    view.srcCombatant and view.srcCombatant.race or {})
	Recount2.Widgets:SetListCheckValue("View_SrcFaction", view.srcCombatant and view.srcCombatant.faction or {})
	Recount2.Widgets:SetListCheckValue("View_SrcAffiliation", view.srcCombatant and view.srcCombatant.affiliation or {})
	Recount2.Widgets:SetListCheckValue("View_SrcReaction", view.srcCombatant and view.srcCombatant.reaction or {})

	Recount2.Widgets:SetListCheckValue("View_DstClass",   view.dstCombatant and view.dstCombatant.class or {})
	Recount2.Widgets:SetListCheckValue("View_DstRace",    view.dstCombatant and view.dstCombatant.race or {})
	Recount2.Widgets:SetListCheckValue("View_DstFaction", view.dstCombatant and view.dstCombatant.faction or {})
	Recount2.Widgets:SetListCheckValue("View_DstAffiliation", view.dstCombatant and view.dstCombatant.affiliation or {})
	Recount2.Widgets:SetListCheckValue("View_DstReaction", view.dstCombatant and view.dstCombatant.reaction or {})

	Recount2.Widgets:SetCompositeCheckValue("View_Info", view.infoFilter or 0)
	Recount2.Widgets:SetCompositeCheckValue("View_Type", view.typeFilter or 0)
	Recount2.Widgets:SetCompositeCheckValue("View_School", view.schoolFilter or 0)
	Recount2.Widgets:SetCompositeCheckValue("View_Power", view.powerFilter or 0)

	Recount2.Widgets:SetRadioValue("View_GroupOn", view.groupOn or "Source GUID")
end


function Recount2_Configs:ConfigView(view)
	if ViewConfig==nil then
		ViewConfig = Recount2.Widgets:CreateWindow("Recount2_ConfigView", 320, 320)
		ViewConfig:SetTitle("Configuring View")
		ViewConfig:SetPoint("CENTER", UIParent, "CENTER")
		ViewConfig:MakeUnresizable()

		ViewConfig.Tabs = Recount2.Widgets:CreateTabs(ViewConfig.innerFrame, ViewConfig.innerFrame:GetWidth()+2, 16)
		ViewConfig.Tabs:SetPoint("TOP", ViewConfig.innerFrame, "TOP")

		ViewConfig.saveButton = Recount2.Widgets:CreateButton(ViewConfig.innerFrame, 60, 16, "Save", SaveView)
		ViewConfig.saveButton:SetPoint("BOTTOMRIGHT", ViewConfig.innerFrame, "BOTTOMRIGHT", -70, 4)

		ViewConfig.revertButton = Recount2.Widgets:CreateButton(ViewConfig.innerFrame, 60, 16, "Revert", LoadView)
		ViewConfig.revertButton:SetPoint("BOTTOMRIGHT", ViewConfig.innerFrame, "BOTTOMRIGHT", -5, 4)

		CreateMainTab()
		CreateFilterTab()
		CreateSourceTab()
		CreateDestinationTab()
		CreateGroupingTab()
	end

	CurrentView = view
	LoadView()

	ViewConfig:Show()
end
