local Recount2 = Recount2
local SM = LibStub:GetLibrary("LibSharedMedia-3.0")
local Recount2_Widgets = Recount2.Widgets

local function SetFontSize(string, size)
	local Font, Height, Flags = string:GetFont()
	string:SetFont(Font, size, Flags)
end






local function Scroll_SetValue(self, v)
	self.slider:SetValue(v)
end

local function Scroll_GetValue(self)
	return self.slider:GetValue()
end

local function Scroll_SetMinMaxValues(self, min, max)
	self.slider:SetMinMaxValues(min, max)

	local v = self.slider:GetValue()
	v = (v-min)/(max-min)
	
	if self.slider.horizontal then
		v=v*self.slider.kWidth
		self.slider.scrollKnobColor:SetPoint("LEFT", self.slider, "LEFT", v, 0)
		self.slider.scrollKnobHighlight:SetPoint("LEFT", self.slider, "LEFT", v, 0)
	else
		v=v*self.slider.kHeight
		self.slider.scrollKnobColor:SetPoint("TOP", self.slider, "TOP", 0, -v)
		self.slider.scrollKnobHighlight:SetPoint("TOP", self.slider, "TOP", 0, -v)
	end
end

local function Scroll_OnValueChanged(self, v)
	if self.OnChange then
		self.OnChange(self.callTo, v)
	end

	local min, max = self:GetMinMaxValues()
	v = (v-min)/(max-min)

	if self.horizontal then
		v=v*self.kWidth
		self.scrollKnobColor:SetPoint("LEFT", self, "LEFT", v, 0)
		self.scrollKnobHighlight:SetPoint("LEFT", self, "LEFT", v, 0)
	else
		v=v*self.kHeight
		self.scrollKnobColor:SetPoint("TOP", self, "TOP", 0, -v)
		self.scrollKnobHighlight:SetPoint("TOP", self, "TOP", 0, -v)
	end
end

local function Scroll_Decrease(self)
	self.slider:SetValue(self.slider:GetValue()-4)
end

local function Scroll_Increase(self)
	self.slider:SetValue(self.slider:GetValue()+4)
end


local function Scroll_SetOnChange(self, func, callTo)
	self.slider.OnChange = func
	self.slider.callTo = callTo
end

local function Scroll_OnSizeChanged(self)
	local w = self:GetRight()-self:GetLeft()
	local h = self:GetTop()-self:GetBottom()
	local slider = self.slider
	local min, max = self.slider:GetMinMaxValues()
	local v = slider:GetValue()
	v = (v-min)/(max-min)
	
	if slider.horizontal then
		slider.kWidth = w-32-h
		slider.scrollKnobColor:SetHeight(h*2)
		slider.scrollKnobHighlight:SetHeight(h*2)

		v=v*slider.kWidth
		slider.scrollKnobColor:SetPoint("LEFT", slider, "LEFT", v, 0)
		slider.scrollKnobHighlight:SetPoint("LEFT", slider, "LEFT", v, 0)
	else
		slider.kHeight = h-32-w
		slider.scrollKnobColor:SetWidth(w*2)
		slider.scrollKnobHighlight:SetWidth(w*2)

		v=v*self.slider.kHeight
		self.slider.scrollKnobColor:SetPoint("TOP", slider, "TOP", 0, -v)
		self.slider.scrollKnobHighlight:SetPoint("TOP", slider, "TOP", 0, -v)
	end
end

function Recount2_Widgets:CreateScrollbar(parent, width, height, orientation)
	local scrollbar = CreateFrame("Frame", nil, parent)
	--local color = {0.3, 0.4, 1.0}
	--local highlight = 0.5
	local color = {0.0, 0.0, 0.0}
	local highlight = 1.0
	local colorTextures = {"Interface\\Addons\\Recount2\\Textures\\scrollbutton_LunaC.tga", "Interface\\Addons\\Recount2\\Textures\\scrollbar_hKnobLunaC.tga", "Interface\\Addons\\Recount2\\Textures\\scrollbar_vKnobLunaC.tga"}
	local highlightTextures = {"Interface\\Addons\\Recount2\\Textures\\scrollbutton_LunaH.tga", "Interface\\Addons\\Recount2\\Textures\\scrollbar_hKnobLunaH.tga", "Interface\\Addons\\Recount2\\Textures\\scrollbar_vKnobLunaH.tga"}

	scrollbar:SetWidth(width)
	scrollbar:SetHeight(height)

	scrollbar.background = scrollbar:CreateTexture(nil, "BACKGROUND")
	scrollbar.background:SetTexture(SM:Fetch("statusbar", "Minimalist"))
	
	
	local slider = CreateFrame("Slider", nil, scrollbar)

	if orientation==nil then
		orientation="HORIZONTAL"
	end

	slider:SetOrientation(orientation)
	if orientation=="HORIZONTAL" then
		slider:SetThumbTexture("Interface\\Addons\\Recount2\\Textures\\transparent.tga")
		slider:SetPoint("TOPLEFT", scrollbar, "TOPLEFT", 8, 0)
		slider:SetPoint("BOTTOMRIGHT", scrollbar, "BOTTOMRIGHT", -9, 0)
		slider.horizontal = true
		slider.kWidth = width-32-height

		local leftButton = CreateFrame("FRAME", nil, scrollbar)
		leftButton:SetWidth(16)
		leftButton.slider = slider
		leftButton.Decrease = Scroll_Decrease
		leftButton:SetPoint("TOPLEFT", scrollbar, "TOPLEFT")
		leftButton:SetPoint("BOTTOMLEFT", scrollbar, "BOTTOMLEFT")
		leftButton:SetFrameLevel(slider:GetFrameLevel()+1)
		leftButton:EnableMouse()
		leftButton:SetScript("OnMouseDown", leftButton.Decrease)

		local rightButton = CreateFrame("FRAME", nil, scrollbar)
		rightButton:SetWidth(16)
		rightButton.slider = slider
		rightButton.Increase = Scroll_Increase
		rightButton:SetPoint("TOPRIGHT", scrollbar, "TOPRIGHT")
		rightButton:SetPoint("BOTTOMRIGHT", scrollbar, "BOTTOMRIGHT")
		rightButton:SetFrameLevel(slider:GetFrameLevel()+1)
		rightButton:EnableMouse()
		rightButton:SetScript("OnMouseDown", rightButton.Increase)

		scrollbar.background:SetPoint("TOPLEFT", scrollbar, "TOPLEFT", 16, 0)
		scrollbar.background:SetPoint("BOTTOMRIGHT", scrollbar, "BOTTOMRIGHT", -16, 0)

		scrollbar.rightButtonColor = scrollbar:CreateTexture(nil, "BORDER")
		scrollbar.rightButtonColor:SetWidth(16)
		scrollbar.rightButtonColor:SetPoint("TOPRIGHT", scrollbar, "TOPRIGHT")
		scrollbar.rightButtonColor:SetPoint("BOTTOMRIGHT", scrollbar, "BOTTOMRIGHT")
		scrollbar.rightButtonColor:SetTexture(colorTextures[1])
		scrollbar.rightButtonColor:SetVertexColor(color[1], color[2], color[3])
		scrollbar.rightButtonColor:SetTexCoord(0.25, 0.5, 0, 1)

		scrollbar.rightButtonHighlight = scrollbar:CreateTexture(nil, "ARTWORK")
		scrollbar.rightButtonHighlight:SetWidth(16)
		scrollbar.rightButtonHighlight:SetPoint("TOPRIGHT", scrollbar, "TOPRIGHT")
		scrollbar.rightButtonHighlight:SetPoint("BOTTOMRIGHT", scrollbar, "BOTTOMRIGHT")
		scrollbar.rightButtonHighlight:SetTexture(highlightTextures[1])
		scrollbar.rightButtonHighlight:SetVertexColor(1.0, 1.0, 1.0, highlight)
		scrollbar.rightButtonHighlight:SetBlendMode("ADD")
		scrollbar.rightButtonHighlight:SetTexCoord(0.25, 0.5, 0, 1)

		scrollbar.leftButtonColor = scrollbar:CreateTexture(nil, "BORDER")
		scrollbar.leftButtonColor:SetWidth(16)
		scrollbar.leftButtonColor:SetPoint("TOPLEFT", scrollbar, "TOPLEFT")
		scrollbar.leftButtonColor:SetPoint("BOTTOMLEFT", scrollbar, "BOTTOMLEFT")
		scrollbar.leftButtonColor:SetTexture(colorTextures[1])
		scrollbar.leftButtonColor:SetVertexColor(color[1], color[2], color[3])
		scrollbar.leftButtonColor:SetTexCoord(0, 0.25, 0, 1)

		scrollbar.leftButtonHighlight = scrollbar:CreateTexture(nil, "ARTWORK")
		scrollbar.leftButtonHighlight:SetWidth(16)
		scrollbar.leftButtonHighlight:SetPoint("TOPLEFT", scrollbar, "TOPLEFT")
		scrollbar.leftButtonHighlight:SetPoint("BOTTOMLEFT", scrollbar, "BOTTOMLEFT")
		scrollbar.leftButtonHighlight:SetTexture(highlightTextures[1])
		scrollbar.leftButtonHighlight:SetVertexColor(1.0, 1.0, 1.0, highlight)
		scrollbar.leftButtonHighlight:SetBlendMode("ADD")
		scrollbar.leftButtonHighlight:SetTexCoord(0, 0.25, 0, 1)

		slider.scrollKnobColor = slider:CreateTexture(nil, "BORDER")
		slider.scrollKnobColor:SetWidth(32)
		slider.scrollKnobColor:SetHeight(height*2)
		slider.scrollKnobColor:SetTexture(colorTextures[2])
		slider.scrollKnobColor:SetVertexColor(color[1], color[2], color[3])

		slider.scrollKnobHighlight = slider:CreateTexture(nil, "ARTWORK")
		slider.scrollKnobHighlight:SetWidth(32)
		slider.scrollKnobHighlight:SetHeight(height*2)
		slider.scrollKnobHighlight:SetTexture(highlightTextures[2])
		slider.scrollKnobHighlight:SetVertexColor(1.0, 1.0, 1.0, highlight)
		slider.scrollKnobHighlight:SetBlendMode("ADD")
	else
		slider:SetThumbTexture("Interface\\Addons\\Recount2\\Textures\\transparent.tga")
		slider:SetPoint("TOPLEFT", scrollbar, "TOPLEFT", 0, -8)
		slider:SetPoint("BOTTOMRIGHT", scrollbar, "BOTTOMRIGHT", 0, 8)
		slider.kHeight = height-32-width

		local downButton = CreateFrame("FRAME", nil, scrollbar)
		downButton:SetHeight(16)
		downButton.slider = slider
		downButton.lockout = 0
		downButton.Increase = Scroll_Increase
		downButton:SetPoint("BOTTOMLEFT", scrollbar, "BOTTOMLEFT")
		downButton:SetPoint("BOTTOMRIGHT", scrollbar, "BOTTOMRIGHT")
		downButton:SetFrameLevel(slider:GetFrameLevel()+1)
		downButton:EnableMouse()
		downButton:SetScript("OnMouseDown", downButton.Increase)

		local upButton = CreateFrame("FRAME", nil, scrollbar)
		upButton:SetHeight(16)
		upButton.slider = slider
		upButton.lockout = 0
		upButton.Decrease = Scroll_Decrease
		upButton:SetPoint("TOPLEFT", scrollbar, "TOPLEFT")
		upButton:SetPoint("TOPRIGHT", scrollbar, "TOPRIGHT")
		upButton:SetFrameLevel(slider:GetFrameLevel()+1)
		upButton:EnableMouse()
		upButton:SetScript("OnMouseDown", upButton.Decrease)

		scrollbar.background:SetPoint("TOPLEFT", scrollbar, "TOPLEFT", 0, -16)
		scrollbar.background:SetPoint("BOTTOMRIGHT", scrollbar, "BOTTOMRIGHT", 0, 16)
		scrollbar.background:SetTexCoord(0, 0, 1, 0, 0, 1, 1, 1)

		scrollbar.upButtonColor = scrollbar:CreateTexture(nil, "BORDER")
		scrollbar.upButtonColor:SetHeight(16)
		scrollbar.upButtonColor:SetPoint("TOPLEFT", scrollbar, "TOPLEFT")
		scrollbar.upButtonColor:SetPoint("TOPRIGHT", scrollbar, "TOPRIGHT")
		scrollbar.upButtonColor:SetTexture(colorTextures[1])
		scrollbar.upButtonColor:SetVertexColor(color[1], color[2], color[3])
		scrollbar.upButtonColor:SetTexCoord(0.75, 1.0, 0, 1)

		scrollbar.upButtonHighlight = scrollbar:CreateTexture(nil, "ARTWORK")
		scrollbar.upButtonHighlight:SetHeight(16)
		scrollbar.upButtonHighlight:SetPoint("TOPLEFT", scrollbar, "TOPLEFT")
		scrollbar.upButtonHighlight:SetPoint("TOPRIGHT", scrollbar, "TOPRIGHT")
		scrollbar.upButtonHighlight:SetTexture(highlightTextures[1])
		scrollbar.upButtonHighlight:SetVertexColor(1.0, 1.0, 1.0, highlight)
		scrollbar.upButtonHighlight:SetBlendMode("ADD")
		scrollbar.upButtonHighlight:SetTexCoord(0.75, 1.0, 0, 1)

		scrollbar.downButtonColor = scrollbar:CreateTexture(nil, "BORDER")
		scrollbar.downButtonColor:SetHeight(16)
		scrollbar.downButtonColor:SetPoint("BOTTOMLEFT", scrollbar, "BOTTOMLEFT")
		scrollbar.downButtonColor:SetPoint("BOTTOMRIGHT", scrollbar, "BOTTOMRIGHT")
		scrollbar.downButtonColor:SetTexture(colorTextures[1])
		scrollbar.downButtonColor:SetVertexColor(color[1], color[2], color[3])
		scrollbar.downButtonColor:SetTexCoord(0.5, 0.75, 0, 1)

		scrollbar.downButtonHighlight = scrollbar:CreateTexture(nil, "ARTWORK")
		scrollbar.downButtonHighlight:SetHeight(16)
		scrollbar.downButtonHighlight:SetPoint("BOTTOMLEFT", scrollbar, "BOTTOMLEFT")
		scrollbar.downButtonHighlight:SetPoint("BOTTOMRIGHT", scrollbar, "BOTTOMRIGHT")
		scrollbar.downButtonHighlight:SetTexture(highlightTextures[1])
		scrollbar.downButtonHighlight:SetVertexColor(1.0, 1.0, 1.0, highlight)
		scrollbar.downButtonHighlight:SetBlendMode("ADD")
		scrollbar.downButtonHighlight:SetTexCoord(0.5, 0.75, 0, 1)

		slider.scrollKnobColor = slider:CreateTexture(nil, "BORDER")
		slider.scrollKnobColor:SetWidth(width*2)
		slider.scrollKnobColor:SetHeight(32)
		slider.scrollKnobColor:SetTexture(colorTextures[3])
		slider.scrollKnobColor:SetVertexColor(color[1], color[2], color[3])

		slider.scrollKnobHighlight = slider:CreateTexture(nil, "ARTWORK")
		slider.scrollKnobHighlight:SetWidth(width*2)
		slider.scrollKnobHighlight:SetHeight(32)
		slider.scrollKnobHighlight:SetTexture(highlightTextures[3])
		slider.scrollKnobHighlight:SetVertexColor(1.0, 1.0, 1.0, highlight)
		slider.scrollKnobHighlight:SetBlendMode("ADD")
	end

	slider.OnValueChanged = Scroll_OnValueChanged
	slider:SetScript("OnValueChanged", slider.OnValueChanged)
	scrollbar.slider = slider

	scrollbar.SetValue = Scroll_SetValue
	scrollbar.GetValue = Scroll_GetValue
	scrollbar.SetMinMaxValues = Scroll_SetMinMaxValues
	scrollbar.SetOnChange = Scroll_SetOnChange
	scrollbar.OnSizeChanged = Scroll_OnSizeChanged

	scrollbar:SetScript("OnSizeChanged", scrollbar.OnSizeChanged)

	scrollbar.Increase = Scroll_Increase
	scrollbar.Decrease = Scroll_Decrease

	scrollbar:SetMinMaxValues(0, 100)
	scrollbar:SetValue(0)

	return scrollbar
end
